/*
 * Copyright (C) 2015 Michal Kosciesza <michal@mkiol.net>
 * 
 * This file is part of Kaktus.
 * 
 * Kaktus is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Kaktus is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Kaktus.  If not, see <http://www.gnu.org/licenses/>.
 */

import bb.cascades 1.2

Page {
    id: root
    
    titleBar: TitleBar {
        title: qsTr("Changelog")
    }
    
    shortcuts: [
        Shortcut {
            key: "Backspace"
            onTriggered: {
                if (nav.count() > 1)
                    nav.pop();
            }
        },
        Shortcut {
            key: "Delete"
            onTriggered: {
                if (nav.count() > 1)
                    nav.pop();
            }
        }
    ]
    
    Container {
        ScrollView {
            Container {
                
                Header {
                    title: qsTr("Version %1").arg("2.5.0")
                }
                
                Container {
                    leftPadding: utils.du(2)
                    rightPadding: utils.du(2)
                    topPadding: utils.du(2)
                    bottomPadding: utils.du(2)
                    
                    LogItem {
                        title: 'Reader View'
                        description: "Reader View is a feature that strips away clutter like buttons, ads and background images, and changes the page's layout for better readability. Reader View implementation in Kaktus is based on Readability.js library, the same that is used in Firefox browser.";
                    }
                    
                    LogItem {
                        title: 'Unsynced data indicator'
                        description: "When Kaktus has any unsynchronized data, indicator (red dot) is shown on the sync buttom in the top bar.";
                    }
                    
                    LogItem {
                        title: 'Smoother offline mode'
                        description: "A few bugs were fixed and general offline mode experience has been improved.";
                    }
                    
                    LogItem {
                        title: 'List filtering'
                        description: "List of articles can be filtered to display all articles, unread and saved or only unread.";
                    }
                    
                    LogItem {
                        title: 'Auto network mode'
                        description: "Option to automatically enabling offline mode on network connection lost.";
                    }
                    
                    LogItem {
                        title: 'Open link behaviour'
                        description: "New option to change how navigation is handled inside built-in web viewer. Hyperlinks could be disabled, opened in an external browser or opened inside web viewer.";
                    }
                }
                
                Header {
                    title: qsTr("Version %1").arg("2.3.1")
                }
                
                Container {
                    leftPadding: utils.du(2)
                    rightPadding: utils.du(2)
                    topPadding: utils.du(2)
                    bottomPadding: utils.du(2)
                    
                    LogItem {
                        title: 'Updated Netvibes API'
                        description: "Fixes to enable updated Netvibes API.";
                    }
                }
                
                Header {
                    title: qsTr("Version %1").arg("2.3")
                }
                
                Container {
                    leftPadding: utils.du(2)
                    rightPadding: utils.du(2)
                    topPadding: utils.du(2)
                    bottomPadding: utils.du(2)
                    
                    LogItem {
                        title: 'Full content from the RSS feed'
                        description: "New option to set 'click on article' action. Following actions are possible: Open article in the build-in viewer, Open article in the external browser, Show full content from the RSS feed.";
                    }
                    
                    LogItem {
                        title: 'Keyboard shortcuts'
                        description: "New keyboard shortcuts has been introduced. For instance, 'S' to save/star article, 'R' to enable read-mode and Backspace for back navigation.";
                    }
                }
                
                Header {
                    title: qsTr("Version %1").arg("2.2")
                }
                
                Container {
                    leftPadding: utils.du(2)
                    rightPadding: utils.du(2)
                    topPadding: utils.du(2)
                    bottomPadding: utils.du(2)
                    
                    LogItem {
                        title: 'Open articles in an external browser'
                        description: "Context menu option to open article directly in an external web browser instead built-in web viewer.";
                    }
                    
                    LogItem {
                        title: 'Updated Netvibes API support'
                        description: "Fixes for updated NV API including, long-awaited, SSL support.";
                    }
                }
                
                Header {
                    title: qsTr("Version %1").arg("2.1")
                }
                
                Container {
                    leftPadding: utils.du(2)
                    rightPadding: utils.du(2)
                    topPadding: utils.du(2)
                    bottomPadding: utils.du(2)
                    
                    LogItem {
                        title: 'Landscape orientation support'
                        description: "Landscace orientation is now supported. There is a new settings option to set specific orientation mode. Possible values: Dynamic, Portrait, Landscape";
                    }
                    
                    LogItem {
                        title: 'Welcome guide'
                        description: "When first running Kaktus, the welcome guide will be shown. It contains information how to navigate the new UI elements like tabs, view modes and online/offline switcher.";
                    }
                    
                    LogItem {
                        title: 'Sync read articles'
                        description: "Previously, unread as well as read articles were synced. With this new option comes possiblity to disable syncing read articles. It will speed up synchronization, but read articles will not be accessible form Kaktus.";
                    }

                    LogItem {
                        title: 'Sort order'
                        description: 'New settings option enabling specific sort order for list of articles. Possible values: Recent first, Oldest first.'
                    }
                    
                    LogItem {
                        title: 'Mark above as read'
                        description: 'New context menu option for marking all above articles as read.'
                    }
                    
                    LogItem {
                        title: 'Old Reader: Like & Liked articles tab'
                        description: 'New context menu option to Like/Unlike article. So called "Slow" tab is now replaced by "Liked" articles tab.'
                    }
                    
                    LogItem {
                        title: 'Old Reader: Social features'
                        description: 'New option to enable/disable Old Reader\'s social features. If enabled, following features will be visible: Following folder, Sharing article with followers, Like/Unlike option, Liked articles tab.'
                    }
                }
                Header {
                    title: qsTr("Version %1").arg("2.0")
                }
                
                Container {
                    leftPadding: utils.du(2)
                    rightPadding: utils.du(2)
                    topPadding: utils.du(2)
                    bottomPadding: utils.du(2)
                    
                    LogItem {
                        title: 'Old Reader support'
                        description: 'Old Reader is supported as new feed aggreagator.'
                    }

                    LogItem {
                        title: 'Many small improvements and bug fixes'
                        description: 'Many improvements, like performance optimization and UI polishing were made.'
                    }
                }
                
                /*Header {
                    title: qsTr("Version %1").arg("1.3")
                }
                
                Container {
                    leftPadding: utils.du(2)
                    rightPadding: utils.du(2)
                    topPadding: utils.du(2)
                    bottomPadding: utils.du(2)
                    
                    LogItem {
                        title: 'Read mode'
                        description: 'When Read mode is enabled, web pages will be reformatted into an easy to read version. '+
                        'All of a website\'s native styles will be striped so you can focus on what you\'re reading. '+
                        'You can switch to Read mode using action on web viewer\'s toolbar.'
                    }
                    
                    LogItem {
                        title: 'Copy & Share URL actions'
                        description: 'New actions were added to web viewer\'s toolbar. Allows you to copy page\'s URL to clipboard or share.'
                    }
                    
                    LogItem {
                        title: 'Sign in with Twitter or Facebook'
                        description: 'In addition to Netvibes credentials, sign in can be done also with Twitter or Facebook account.'
                    }
                                
                    LogItem {
                        title: 'Auto-hide bars & many other UI improvements'
                        description: 'Many UI improvements were made. E.g. title bar and action bar are auto-hidden.'
                    }
                }
                
                Header {
                    title: qsTr("Version %1").arg("1.2.3")
                }

                Container {
                    leftPadding: utils.du(2)
                    rightPadding: utils.du(2)
                    topPadding: utils.du(2)
                    bottomPadding: utils.du(2)

                    LogItem {
                        title: 'Initial version for BlackBerry'
                        description: 'This is first version of Kaktus for BlackBerry 10.'
                    }
                }*/
            }
        }

    }
}
