/*
 * Copyright (C) 2015 Michal Kosciesza <michal@mkiol.net>
 * 
 * This file is part of Kaktus.
 * 
 * Kaktus is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Kaktus is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Kaktus.  If not, see <http://www.gnu.org/licenses/>.
 */

import bb.cascades 1.2

Page {

    id: root

    titleBar: TitleBar {
        title: qsTr("Settings")
    }

    property bool dark: Application.themeSupport.theme.colorTheme.style == VisualStyle.Dark

    function disconnectSignals() {
    }

    attachedObjects: [
        SignOutDialog {
            id: signOutDialog
            onOk: {
                settings.signedIn = false;
            }
        }
    ]
    
    shortcuts: [
        Shortcut {
            key: "Backspace"
            onTriggered: {
                if (nav.count() > 1)
                    nav.pop();
            }
        },
        Shortcut {
            key: "Delete"
            onTriggered: {
                if (nav.count() > 1)
                    nav.pop();
            }
        }
    ]

    Container {

        ScrollView {
            Container {
                Header {
                    title: app.isNetvibes ? "Netvibes":
                           app.isOldReader ? "Old Reader" : "Feedly"
                }

                Container {
                    leftPadding: utils.du(2)
                    rightPadding: utils.du(2)
                    topPadding: utils.du(2)
                    bottomPadding: utils.du(2)

                    TextLabel {
                        text: settings.signedIn ? qsTr("Signed in with") : qsTr("Not signed in")
                        value: settings.signedIn ?
                                settings.signinType==0 ? settings.getUsername() :
                                settings.signinType==1 ? "Twitter" :
                                settings.signinType==2 ? "Facebook" :
                                settings.signinType==10 ? settings.getUsername() :
                                settings.signinType==20 ? settings.getProvider() : "" : ""
                        enabled: settings.signedIn
                        buttonText: qsTr("Sign out")
                        onClicked: {
                            signOutDialog.show();
                        }
                    }

                    TextLabel {
                        text: settings.signedIn && utils.defaultDashboardName() !== "" ? qsTr("Dashboard in use") : qsTr("Dashboard not selected")
                        value: settings.signedIn && utils.defaultDashboardName() !== "" ? utils.defaultDashboardName() : ""
                        buttonText: settings.signedIn ? qsTr("Change") : ""
                        visible: app.isNetvibes
                        onClicked: {
                            utils.setDashboardModel();
                            nav.push(dashboardPage.createObject());
                        }
                    }
                }

                Header {
                    title: qsTr("Syncronization")
                }

                Container {
                    leftPadding: utils.du(2)
                    rightPadding: utils.du(2)
                    topPadding: utils.du(2)
                    bottomPadding: utils.du(2)

                    DropDown {
                        visible: enabled
                        title: qsTr("Sync timeframe")
                        enabled: app.isOldReader || app.isFeedly

                        selectedIndex: {
                            var retention = settings.getRetentionDays();
                            if (retention < 1)
                                return 5;
                            if (retention < 3)
                                return 0;
                            if (retention < 7)
                                return 1;
                            if (retention < 14)
                                return 2;
                            if (retention < 30)
                                return 3;
                            return 4;
                        }

                        options: [
                            Option {
                                value: 1
                                text: qsTr("1 Day")
                            },
                            Option {
                                value: 3
                                text: qsTr("3 Days")
                            },
                            Option {
                                value: 7
                                text: qsTr("1 Week")
                            },
                            Option {
                                value: 14
                                text: qsTr("2 Weeks")
                            },
                            Option {
                                value: 30
                                enabled: !utils.isLight()
                                text: enabled ? qsTr("1 Month") : qsTr("1 Month (only in pro edition)")
                            },
                            Option {
                                value: 0
                                enabled: !utils.isLight()
                                text: enabled ? qsTr("Wide as possible") : qsTr("Wide as possible (only in pro edition)")
                            }
                        ]

                        onSelectedOptionChanged: {
                            settings.setRetentionDays(selectedOption.value);
                        }

                    }

                    Label {
                        visible: app.isOldReader || app.isFeedly
                        multiline: true
                        textStyle.base: SystemDefaults.TextStyles.SubtitleText
                        textStyle.color: utils.secondaryText()
                        text: qsTr("Most recent articles will be syncronized according to the defined timeframe.") + " " +
                        (settings.signinType < 20 ? qsTr("Regardless of the value, all starred, liked and shared items will be synced as well.") : qsTr("Regardless of the value, all saved items will be synced as well.")) + " " +
                        qsTr("Be aware, this parameter has significant impact on the speed of synchronization.")
                    }
                    
                    ToggleComponent {
                        text: qsTr("Sync read articles")
                        description: qsTr("In addition to unread also read articles will be synced. Disabling this option will speed up synchronization, but read articles will not be accessible form Kaktus.")
                        checked: settings.syncRead
                        onCheckedChanged: {
                            settings.syncRead = checked;
                        }
                    }
                }
                
                Header {
                    title: qsTr("Cache")
                }

                Container {
                    leftPadding: utils.du(2)
                    rightPadding: utils.du(2)
                    topPadding: utils.du(2)
                    bottomPadding: utils.du(2)

                    TextLabel {
                        text: qsTr("Current cache size")
                        value: utils.getHumanFriendlySizeString(dm.cacheSize)
                        buttonText: dm.cacheSize > 0 ? qsTr("Delete cache") : ""
                        onClicked: {
                            //notification.show(qsTr("Removing cache..."));
                            fetcher.cancel();
                            dm.cancel();
                            dm.removeCache();
                        }
                    }

                    DropWithDescription {
                        title: qsTr("Network mode")
                        description: qsTr("In the offline mode, Kaktus will only use local cache to get web pages and images, so "+
                        "network connection won't be needed.")
                        options: [
                            Option {
                                id: defaultNetworkModeOption
                                selected: ! settings.offlineMode
                                imageSource: dark ? "asset:///network-online.png" : "asset:///network-onlined.png"
                                value: false
                                text: qsTr("Online")
                            },
                            Option {
                                selected: settings.offlineMode
                                imageSource: dark ? "asset:///network-offline.png" : "asset:///network-offlined.png"
                                value: true
                                text: enabled ? qsTr("Offline") : qsTr("Offline (only in pro edition)")
                                enabled: ! utils.isLight()
                            }
                        ]
                        onSelectedOptionChanged: {
                            settings.offlineMode = selectedOption.value;
                        }

                    }
                    
                    ToggleComponent {
                        text: qsTr("Auto network mode")
                        description: qsTr("Network mode will be switched automatically on network connection lost or restore") + 
                                    (utils.isLight() ? " (only in pro edition)." : ".")
                        checked: settings.autoOffline
                        enabled: ! utils.isLight()
                        onCheckedChanged: {
                            settings.autoOffline = checked;
                        }
                    }
                    
                    DropWithDescription {
                        description: qsTr("After sync the content of all items will be downloaded " + "and cached for access in the offline mode.")
                        title: qsTr("Cache articles")
                        options: [
                            Option {
                                selected: settings.cachingMode == 0
                                value: 0
                                text: qsTr("Never")
                            },
                            Option {
                                selected: settings.cachingMode == 1
                                value: 1
                                text: enabled ? qsTr("WiFi only") : qsTr("WiFi only (only in pro edition)")
                                enabled: ! utils.isLight()
                            },
                            Option {
                                selected: settings.cachingMode == 2
                                value: 2
                                text: enabled ? qsTr("Always") : qsTr("Always (only in pro edition)")
                                enabled: ! utils.isLight()
                            }
                        ]
                        onSelectedOptionChanged: {
                            settings.cachingMode = selectedOption.value;
                        }
                    }
                }
                
                Header {
                    title: qsTr("Web viewer")
                }
                
                Container {
                    
                    leftPadding: utils.du(2)
                    rightPadding: utils.du(2)
                    topPadding: utils.du(2)
                    bottomPadding: utils.du(2)
                    
                    DropWithDescription {
                        title: qsTr("Open link behaviour")
                        description: qsTr("Defines how navigation is handled inside built-in web viewer. Hyperlinks could be disabled, opened in an external browser or opened inside web viewer.")
                        options: [
                            Option {
                                selected: settings.webviewNavigation == value
                                value: 0
                                text: qsTr("Disabled")
                            },
                            Option {
                                selected: settings.webviewNavigation == value
                                value: 1
                                imageSource: dark ? "asset:///browser.png" : "asset:///browserd.png"
                                text: qsTr("External browser")
                            },
                            Option {
                                selected: settings.webviewNavigation == value
                                value: 2
                                imageSource: dark ? "asset:///webview.png" : "asset:///webviewd.png"
                                text: qsTr("Web viewer")
                            }
                        ]
                        onSelectedOptionChanged: {
                            settings.webviewNavigation = selectedOption.value;
                        }
                    }
                    
                    ToggleComponent {
                        text: qsTr("Auto switch to Reader View")
                        description: qsTr("Reader View is a feature that strips away clutter like buttons, ads and background images, and changes the page's layout for better readability. By enabling this option, Reader View will be automatically switch on when page is loaded in the web viewer.")
                        iconSource: checked ? dark ? "asset:///reader-selected.png" : "asset:///reader-selectedd.png" : 
                                              dark ? "asset:///reader.png" : "asset:///readerd.png"
                        checked: settings.readerMode
                        
                        onCheckedChanged: {
                            settings.readerMode = checked;
                        }
                    }
                    
                    DropWithDescription {
                        title: qsTr("Reader View theme")
                        description: qsTr("Style of theme which will be used to display articles in Reader View.")
                        options: [
                            Option {
                                selected: settings.readerTheme === value
                                value: String("dark")
                                text: qsTr("Dark")
                            },
                            Option {
                                selected: settings.readerTheme === value
                                value: String("light")
                                text: qsTr("Light")
                            }
                        ]
                        onSelectedOptionChanged: {
                            settings.readerTheme = selectedOption.value;
                        }
                    }
                    
                    /*ToggleComponent {
                        text: qsTr("Auto switch to Night View")
                        description: qsTr("Night View reduces the brightness of websites. By enabling this option, Night View will be automatically switch on when page is loaded in the web viewer.")
                        iconSource: checked ? dark ? "asset:///night-selected.png" : "asset:///night-selectedd.png" : 
                                              dark ? "asset:///night.png" : "asset:///nightd.png" 
                        checked: settings.nightMode
                        
                        onCheckedChanged: {
                            settings.nightMode = checked;
                        }
                    }*/

                    Container {
                        topPadding: utils.du(2)

                        Container {
                            layout: DockLayout {}
                            preferredWidth: display.pixelSize.width
                            Label {
                                horizontalAlignment: HorizontalAlignment.Left
                                text: qsTr("Viewer font size level")
                            }
                            Label {
                                horizontalAlignment: HorizontalAlignment.Right
                                text: Math.floor(fontSizeSlider.value) + "%"
                                onTouch: {
                                    fontSizeSlider.value = 100.0;
                                }
                            }
                        }
                        
                        Container {
                            topPadding: utils.du(1)
                            layout: StackLayout {
                                orientation: LayoutOrientation.LeftToRight
                            }

                            ImageView {
                                imageSource: dark ? "asset:///fontdown.png" : "asset:///fontdownd.png"
                                minWidth: 96
                            }
                        
                            Slider {
                                id: fontSizeSlider
                                property bool created: false
                                fromValue: 50.0
                                toValue: 200.0
                                onValueChanged: {
                                    if (created)
                                        settings.zoom = Math.floor(value)/100;
                                }
                                onCreationCompleted: {
                                    created = true
                                    value = Math.floor(settings.zoom * 100)
                                }
                            }
                            
                            ImageView {
                                imageSource: dark ? "asset:///fontup.png" : "asset:///fontupd.png"
                                minWidth: 96
                            }
                            
                            
                        }
                    }
                }

                Header {
                    title: qsTr("UI")
                }

                Container {

                    leftPadding: utils.du(2)
                    rightPadding: utils.du(2)
                    topPadding: utils.du(2)
                    bottomPadding: utils.du(2)
                    
                    DropDown {
                        title: qsTr("Sort order for list of articles")
                        options: [
                            Option {
                                selected: settings.showOldestFirst == value
                                value: false
                                text: qsTr("Recent first")
                            },
                            Option {
                                selected: settings.showOldestFirst == value
                                value: true
                                text: qsTr("Oldest first")
                            }
                        ]
                        onSelectedOptionChanged: {
                            settings.showOldestFirst = selectedOption.value;
                        }
                    }
                    
                    DropWithDescription {
                        title: qsTr("Clicking on article behaviour")
                        description: qsTr("Defines the behavior for clicking on an article item. Article can be opened in the built-in web viewer, opened in an external browser or full RSS feed content can be shown.")
                        options: [
                            Option {
                                selected: settings.clickBehavior == value
                                imageSource: dark ? "asset:///webview.png" : "asset:///webviewd.png"
                                value: 0
                                text: qsTr("Web viewer")
                            },
                            Option {
                                selected: settings.clickBehavior == value
                                imageSource: dark ? "asset:///browser.png" : "asset:///browserd.png"
                                value: 1
                                text: qsTr("External browser")
                            },
                            Option {
                                selected: settings.clickBehavior == value
                                value: 2
                                imageSource: dark ? "asset:///rss.png" : "asset:///rssd.png"
                                text: qsTr("Feed content")
                            }
                        ]
                        onSelectedOptionChanged: {
                            settings.clickBehavior = selectedOption.value;
                        }
                    }
                    
                    DropWithDescription {
                        title: qsTr("List filtering")
                        description: app.isNetvibes || app.isFeedly ?
                            qsTr("List of articles can be filtered to display all articles, unread and saved or only unread.") :
                            qsTr("List of articles can be filtered to display all articles, unread and starred or only unread.")
                        options: [
                            Option {
                                selected: settings.filter == value
                                value: 0
                                text: qsTr("All articles")
                            },
                            Option {
                                selected: settings.filter == value
                                value: 1
                                text: app.isNetvibes || app.isFeedly ? qsTr("Unread or saved") : qsTr("Unread or starred")
                            },
                            Option {
                                selected: settings.filter == value
                                value: 2
                                text: qsTr("Only unread")
                            }
                        ]
                        onSelectedOptionChanged: {
                            settings.filter = selectedOption.value;
                        }
                    }

                    ToggleComponent {
                        text: qsTr("Show images")
                        checked: settings.showTabIcons

                        onCheckedChanged: {
                            settings.showTabIcons = checked;
                        }
                    }
                    
                    ToggleComponent {
                        text: utils.isLight() ? qsTr("Social features (only in pro edition)") : qsTr("Social features")
                        enabled: app.isOldReader && !utils.isLight()
                        checked: settings.showBroadcast
                        description: qsTr("Following Old Reader's social features will be enabled: Following folder, Sharing article with followers, Like option, Liked tab.")
                        
                        onCheckedChanged: {
                            settings.showBroadcast = checked;
                        }
                        
                        visible: app.isOldReader
                    }

                    DropDown {
                        title: qsTr("Theme")
                        visible: utils.checkOSVersion(10, 3)
                        options: [
                            Option {
                                selected: settings.theme == value
                                value: 1
                                text: qsTr("Bright")
                            },
                            Option {
                                selected: settings.theme == value
                                value: 2
                                text: qsTr("Dark")
                            }
                        ]
                        onSelectedOptionChanged: {
                            if (settings.theme != selectedOption.value)
                                notification.show(qsTr("Changes will take effect after you restart Kaktus."));
                            settings.theme = selectedOption.value;
                        }
                    }
                    
                    DropDown {
                        title: qsTr("Orientation")
                        options: [
                            Option {
                                selected: settings.allowedOrientations == value
                                value: 0
                                text: qsTr("Dynamic")
                            },
                            Option {
                                selected: settings.allowedOrientations == value
                                value: 1
                                text: qsTr("Portrait")
                            },
                            Option {
                                selected: settings.allowedOrientations == value
                                value: 2
                                text: qsTr("Landscape")
                            }
                        ]
                        onSelectedOptionChanged: {
                            settings.allowedOrientations = selectedOption.value;
                        }
                    }
                    
            }

            Header {
                title: qsTr("Other")
            }

            Container {
                horizontalAlignment: HorizontalAlignment.Center

                leftPadding: utils.du(2)
                rightPadding: utils.du(2)
                topPadding: utils.du(2)
                bottomPadding: utils.du(2)

                Button {
                    text: qsTr("Show User Guide")

                    onClicked: {
                        guide.open();
                    }
                }

            }

        }
    }
}
}
